//+------------------------------------------------------------------+
//|                                                           mladen |
//|                                               mladenfx@gmail.com |
//+------------------------------------------------------------------+
#property copyright "copyright mladen"
#property link      "mladenfx@gmail.com"
#property indicator_separate_window
#property indicator_buffers 5
#property indicator_plots   4
#property indicator_label1  "Dynamic zones cfb adaptive"
#property indicator_type1   DRAW_LINE
#property indicator_color1  clrOrange
#property indicator_label2  "Dynamic zones cfb adaptive"
#property indicator_type2   DRAW_LINE
#property indicator_color2  clrSilver
#property indicator_style2  STYLE_DOT
#property indicator_label3  "Dynamic zones cfb adaptive"
#property indicator_type3   DRAW_LINE
#property indicator_color3  clrLimeGreen
#property indicator_label4  "Dynamic zones cfb adaptive"
#property indicator_type4   DRAW_COLOR_LINE
#property indicator_color4  clrLimeGreen,clrOrange
#property indicator_style4  STYLE_SOLID
#property indicator_width4  2



//
//
//
//
//

#import "dynamicZone.dll"
   double dzBuyP(double& sourceArray[],double probabiltyValue, int lookBack, int bars, int i,double precision);
   double dzSellP(double& sourceArray[],double probabiltyValue, int lookBack, int bars, int i,double precision);
#import

//
//
//
//
//

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average     // Average (high+low+oprn+close)/4
};

input int      LowLimit               = 10;  // Low limit
input int      HighLimit              = 20;  // High limit
input double   SmoothLength           = 5;   // Smoothing period
input double   SmoothPhase            = 0;   // Smoothing phase
input enPrices Price                  = pr_close; // Price
input int      CfbNormLength          = 50;  // cfb normalization length
input int      CfbDepth               = 13;  // cfb depth
input enPrices CfbPrice               = pr_weighted; // cfb uses price
input int      CfbSmooth              = 8;   // cfb smooth
input int      CfbResultSmooth        = 5;   // cfb result smoothing
input double   CfbResultPhase         = 0;   // cfb result smoothing phase
input bool     SmoothDouble           = false; // cfb result double smoothing
input int      DzLookBackBars         = 70;  // dynamic zones look back bars
input double   DzStartBuyProbability  = 0.1; // dynamic zones buy probability
input double   DzStartSellProbability = 0.1; // dynamic zones sell probability
input double   ZoneSpacing            = 0;   // zone spacing

input bool   alertsOn              = false;
input bool   alertsOnLevelsBreak   = false;
input bool   alertsOnLevelsRetrace = true;
input bool   alertsOnZeroBreak     = false;
input bool   alertsOnCurrent       = true;
input bool   alertsMessage         = true;
input bool   alertsSound           = false;
input bool   alertsEmail           = false;

input bool   arrowsVisible         = false;
input string arrowsIdentifier      = "velocityAdaptivearrows";
input bool   arrowsOnLevelsBreak   = true;
input bool   arrowsOnLevelsRetrace = false;
input bool   arrowsOnZeroBreak     = false;
input color  arrowsUpColor         = clrLimeGreen;
input color  arrowsDnColor         = clrOrange;
input color  arrowsZeroUpColor     = clrLimeGreen;
input color  arrowsZeroDnColor     = clrOrange;

//
//
//
//
//

double vel[];
double velc[];
double bli[];
double sli[];
double zli[];


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int OnInit()
{
   SetIndexBuffer(0,bli);
   SetIndexBuffer(1,zli);
   SetIndexBuffer(2,sli);
   SetIndexBuffer(3,vel ,INDICATOR_DATA); ArraySetAsSeries(vel,true);
   SetIndexBuffer(4,velc,INDICATOR_COLOR_INDEX); 
   return(0);
}


//
//
//
//
//

void OnDeinit(const int reason)
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal(0,0)-1; i>=0; i--)
   {
      string objectName = ObjectName(0,i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(0,objectName);
   }
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double cfb[];
double prices[];
double trends[][6];
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
{
   double spacing = ZoneSpacing;

   //
   //
   //
   //
   //

   if (ArrayRange(cfb,0)    != rates_total) ArrayResize(cfb   ,rates_total);
   if (ArrayRange(prices,0) != rates_total) ArrayResize(prices,rates_total);
   if (ArrayRange(trends,0) != rates_total) ArrayResize(trends,rates_total);
   for (int i=(int)MathMax(prev_calculated-1,1); i<rates_total; i++)
   {
      cfb[i] = iDSmooth(iCfb(rates_total,getPrice(CfbPrice,open,close,high,low,i,rates_total),CfbDepth,CfbSmooth,i),CfbResultSmooth,CfbResultPhase,rates_total,i);
    
      //
      //
      //
      //
      //
         double cfbMax = cfb[i];
         double cfbMin = cfb[i];
         for (int k=1; k<CfbNormLength && (i-k)>=0; k++ )
            {
               cfbMax = MathMax(cfb[i-k],cfbMax);
               cfbMin = MathMin(cfb[i-k],cfbMin);
            }
            double denominator = cfbMax-cfbMin;
            double ratio       = 0.5;                 
               if (denominator> 0)
                  ratio = (cfb[i]-cfbMin)/denominator;
      //
      //
      //
      //
      //

      double length = MathCeil(LowLimit+ratio*(HighLimit-LowLimit));
      prices[i] = getPrice(Price,open,close,high,low,i,rates_total);
      vel[rates_total-i-1] = iDSmooth(iVEL(length,i),SmoothLength,SmoothPhase,rates_total,i,1);
      bli[i]               = dzBuyP (vel,DzStartBuyProbability,  DzLookBackBars, rates_total, rates_total-i-1, 0.0001);
      sli[i]               = dzSellP(vel,DzStartSellProbability, DzLookBackBars, rates_total, rates_total-i-1, 0.0001);
      zli[i]               = dzSellP(vel,0.5                   , DzLookBackBars, rates_total, rates_total-i-1, 0.0001);
      
      //
      //
      //
      //
      //

      double diff  = MathAbs(bli[i]-zli[i]);
      bool   large = (diff>spacing);
         setTrends(zli,large,0,rates_total,i);
         setTrends(bli,large,1,rates_total,i);
         setTrends(sli,large,2,rates_total,i);
         trends[i][5] = trends[i-1][5];
            if (vel[rates_total-i-1]>vel[rates_total-i]) trends[i][5] =  1;
            if (vel[rates_total-i-1]<vel[rates_total-i]) trends[i][5] = -1;
            velc[i] = velc[i-1];
               if (trends[i][5] ==  1) velc[i] = 0;
               if (trends[i][5] == -1) velc[i] = 1;
      manageArrow(i,time[i],high,low);
   }            
   manageAlerts(time[rates_total-1]);
   return(rates_total);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void setTrends(double& array[],bool large, int forTrend, int bars, int i)
{
   if (i<1) return;
   trends[i][forTrend] = trends[i-1][forTrend];
      if (vel[bars-i-1]>array[i] && large) trends[i][forTrend] =  1;
      if (vel[bars-i-1]<array[i] && large) trends[i][forTrend] = -1;
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void manageArrow(int i,datetime time,const double& high[],const double& low[])
{
   if (arrowsVisible)
   {
      ObjectDelete(0,arrowsIdentifier+":"+(string)time);
      if (arrowsOnZeroBreak && trends[i][0]!=trends[i-1][0])
      {
         if (trends[i][0] ==  1) drawArrow(i,time,high,low,arrowsZeroUpColor,241,false);
         if (trends[i][0] == -1) drawArrow(i,time,high,low,arrowsZeroDnColor,242,true);
      }
      if (trends[i][1]!=trends[i-1][1])
      {
         if (arrowsOnLevelsRetrace && trends[i][1] ==  1) drawArrow(i,time,high,low,arrowsUpColor,241,false);
         if (arrowsOnLevelsBreak   && trends[i][1] == -1) drawArrow(i,time,high,low,arrowsDnColor,242,true);
      }
      if (trends[i][2]!=trends[i-1][2])
      {
         if (arrowsOnLevelsBreak   && trends[i][2] ==  1) drawArrow(i,time,high,low,arrowsUpColor,241,false);
         if (arrowsOnLevelsRetrace && trends[i][2] == -1) drawArrow(i,time,high,low,arrowsDnColor,242,true);
      }
   }
}               

//
//
//
//
//

void drawArrow(int i,datetime time,const double& high[],const double& low[], color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+(string)time;
   double gap  = 0; for (int k=0; k<20 && (i-k)>=0; k++) gap += high[i-k]-low[i-k]; gap /= 20;
   
      //
      //
      //
      //
      //
      
      ObjectCreate(0,name,OBJ_ARROW,0,time,0);
         ObjectSetInteger(0,name,OBJPROP_ARROWCODE,theCode);
         ObjectSetInteger(0,name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSetDouble(0,name,OBJPROP_PRICE,0,high[i]+gap*2);
         else  ObjectSetDouble(0,name,OBJPROP_PRICE,0,low[i] -gap);
}


//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts(datetime time)
{
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      
      //
      //
      //
      //
      //
      
      if (alertsOnZeroBreak)
      {
         static datetime time1 = 0;
         static string   mess1 = "";
            if (alertsOnZeroBreak && trends[whichBar][0] ==  1 && trends[whichBar-1][0] !=  1) doAlert(time,time1,mess1,whichBar,"zero line up");
            if (alertsOnZeroBreak && trends[whichBar][0] == -1 && trends[whichBar-1][0] != -1) doAlert(time,time1,mess1,whichBar,"zero line down");
      }
      if (alertsOnLevelsBreak || alertsOnLevelsRetrace)
      {
         static datetime time2 = 0;
         static string   mess2 = "";
            if (alertsOnLevelsRetrace && trends[whichBar][1] ==  1 && trends[whichBar-1][1] !=  1) doAlert(time,time2,mess2,whichBar,"retraced "+DoubleToString(bli[whichBar],5)+" level up");
            if (alertsOnLevelsBreak   && trends[whichBar][1] == -1 && trends[whichBar-1][1] != -1) doAlert(time,time2,mess2,whichBar,"broke "+DoubleToString(bli[whichBar],5)+" level down");
         static datetime time3 = 0;
         static string   mess3 = "";
            if (alertsOnLevelsRetrace && trends[whichBar][2] == -1 && trends[whichBar-1][2] != -1) doAlert(time,time3,mess3,whichBar,"retraced "+DoubleToString(sli[whichBar],5)+" level down");
            if (alertsOnLevelsBreak   && trends[whichBar][2] ==  1 && trends[whichBar-1][2] !=  1) doAlert(time,time3,mess3,whichBar,"broke "+DoubleToString(sli[whichBar],5)+" level up");
      }            
   }
}

//
//
//
//
//

void doAlert(datetime time,datetime& previousTime, string& previousAlert,  int forBar, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != time) {
       previousAlert  = doWhat;
       previousTime   = time;

       //
       //
       //
       //
       //

       message =  _Symbol+" at "+TimeToString(TimeLocal(),TIME_SECONDS)+" DZ veocity "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(_Symbol+"DZ veocity",message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}
  
//+------------------------------------------------------------------
//|                                                                 
//+------------------------------------------------------------------
//
//
//
//
//

double iVEL(double length,int i)
{
   static int vel_length = -1;
   static double coeff0;
   static double coeff1; 
   static double coeff2; 
   static double coeff3; 
          if (vel_length != length)
          {
              vel_length = (int)length;
                  coeff0 = length + 1;
                  coeff1 = coeff0 * (    coeff0+1.0)/2.0;
                  coeff2 = coeff1 * (2.0*coeff0+1.0)/3.0;
                  coeff3 = coeff1 * coeff1 * coeff1 - coeff2 * coeff2; 
          }
   
   //
   //
   //
   //
   //

   double suma = 0.0;
   double sumb = 0.0;
      for(int l=0; l<=length && (i-l)>=0; l++)
      {
         suma += prices[i-l] * (coeff0-l);
         sumb += prices[i-l] * (coeff0-l) * (coeff0-l);
      }
   double _vel = (sumb*coeff1 - suma*coeff2) / (coeff3*_Point); 
   return(_vel);
}


//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

int    Depths[] = {2,3,4,6,8,12,16,24,32,48,64,96,128,192};
double workCfb[][28];

//
//
//
//
//

double iCfb(int totalBars, double price, int depth, int smooth, int i)
{
   if (ArrayRange(workCfb,0) != totalBars) ArrayResize(workCfb,totalBars); depth = MathMax(MathMin(depth,13),0);
         
   //
   //
   //
   //
   //

   double suma     = 0;
   double sumb     = 0;
   double _cfb      = 0;
   double evenCoef = 1;
   double oddCoef  = 1;
   double avg      = 0;
         
      if (i>=smooth)
         for (int k=depth-1; k>=0; k--)
         {
            workCfb[i][k]    = iCfbFunc(totalBars,price,i,Depths[k],k);
            workCfb[i][k+14] = workCfb[i-1][k+14] + (workCfb[i][k]-workCfb[i-smooth][k])/smooth;

                  if ((k%2)==0)
                        { avg = oddCoef  * workCfb[i][k+14]; oddCoef  = oddCoef  * (1 - avg); }
                  else  { avg = evenCoef * workCfb[i][k+14]; evenCoef = evenCoef * (1 - avg); }
               
               suma += avg*avg*Depths[k];
               sumb += avg;
         }
      else for (int k=depth-1; k>=0; k--) { workCfb[i][k] = 0; workCfb[i][k+14] = 0; }            

   //
   //
   //
   //
   //

   if (sumb != 0) _cfb = suma/sumb;
   return(_cfb);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

double  workCfbFunc[][70];
#define _prices 0
#define _roc    1
#define _value1 2
#define _value2 3
#define _value3 4

//
//
//
//

double iCfbFunc(int totalBars, double price, int r, int depth, int k)
{
   k *= 5;
      if (ArrayRange(workCfbFunc,0) != totalBars) ArrayResize(workCfbFunc,totalBars);
      if (r<=(depth+1))
      {
         workCfbFunc[r][k+_prices] = 0;
         workCfbFunc[r][k+_roc]    = 0;
         workCfbFunc[r][k+_value1] = 0;
         workCfbFunc[r][k+_value2] = 0;
         workCfbFunc[r][k+_value3] = 0;
         return(0);
      }         
      workCfbFunc[r][k+_prices] = price; 

   //
   //
   //
   //
   //

      workCfbFunc[r][k+_roc]    = MathAbs(workCfbFunc[r][k+_prices] - workCfbFunc[r-1][k+_prices]);
      workCfbFunc[r][k+_value1] = workCfbFunc[r-1][k+_value1] - workCfbFunc[r-depth][k+_roc] + workCfbFunc[r][k+_roc];
      workCfbFunc[r][k+_value2] = workCfbFunc[r-1][k+_value2] - workCfbFunc[r-1][k+_value1] + workCfbFunc[r][k+_roc]*depth;
      workCfbFunc[r][k+_value3] = workCfbFunc[r-1][k+_value3] - workCfbFunc[r-1-depth][k+_prices] + workCfbFunc[r-1][k+_prices];
   
      double dividend = MathAbs(depth*workCfbFunc[r][k+_prices]-workCfbFunc[r][k+_value3]);

      //
      //
      //
      //
      //
         
   if (workCfbFunc[r][k+_value2] != 0)
         return( dividend / workCfbFunc[r][k+_value2]);
   else  return(0.00);            
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

double wrk[][40];

#define bsmax  5
#define bsmin  6
#define volty  7
#define vsum   8
#define avolty 9

//
//
//
//
//

double iDSmooth(double price, double length, double phase, int bars, int i, int s=0)
{
   if (SmoothDouble)
         return (iSmooth(iSmooth(price,MathSqrt(length),phase,bars,i,s*2),MathSqrt(length),phase,bars,i,s*2+1));
   else  return (iSmooth(price,length,phase,bars,i,s*2));
}

//
//
//
//
//

double iSmooth(double price, double length, double phase, int bars, int r, int s=0)
{
   if (length <=1 || bars<1) return(price);
   if (ArrayRange(wrk,0) != bars) ArrayResize(wrk,bars); s*= 10;
   
      if (r<3) { int k; for(k=0; k<7; k++) wrk[r][k+s]=price; for(; k<10; k++) wrk[r][k+s]=0; return(price); }
 
   //
   //
   //
   //
   //
   
      double len1   = MathMax(MathLog(MathSqrt(0.5*(length-1)))/MathLog(2.0)+2.0,0);
      double pow1   = MathMax(len1-2.0,0.5);
      double del1   = price - wrk[r-1][bsmax+s];
      double del2   = price - wrk[r-1][bsmin+s];
      double div    = 1.0/(10.0+10.0*(MathMin(MathMax(length-10,0),100))/100);
      int    forBar = (int)MathMin(r,10);
	
         wrk[r][volty+s] = 0;
               if(MathAbs(del1) > MathAbs(del2)) wrk[r][volty+s] = MathAbs(del1); 
               if(MathAbs(del1) < MathAbs(del2)) wrk[r][volty+s] = MathAbs(del2); 
         wrk[r][vsum+s] =	wrk[r-1][vsum+s] + (wrk[r][volty+s]-wrk[r-forBar][volty+s])*div;
         
         //
         //
         //
         //
         //
   
         wrk[r][avolty+s] = wrk[r-1][avolty+s]+(2.0/(MathMax(4.0*length,30)+1.0))*(wrk[r][vsum+s]-wrk[r-1][avolty+s]);
         double dVolty = 0;   
            if (wrk[r][avolty+s] > 0)
               dVolty = wrk[r][volty+s]/wrk[r][avolty+s];
	               if (dVolty > MathPow(len1,1.0/pow1)) dVolty = MathPow(len1,1.0/pow1);
                  if (dVolty < 1)                      dVolty = 1.0;

      //
      //
      //
      //
      //
	        
   	double pow2 = MathPow(dVolty, pow1);
      double len2 = MathSqrt(0.5*(length-1))*len1;
      double Kv   = MathPow(len2/(len2+1), MathSqrt(pow2));

         if (del1 > 0) wrk[r][bsmax+s] = price; else wrk[r][bsmax+s] = price - Kv*del1;
         if (del2 < 0) wrk[r][bsmin+s] = price; else wrk[r][bsmin+s] = price - Kv*del2;
	
   //
   //
   //
   //
   //
      
      double R     = MathMax(MathMin(phase,100),-100)/100.0 + 1.5;
      double beta  = 0.45*(length-1)/(0.45*(length-1)+2);
      double alpha = MathPow(beta,pow2);

         wrk[r][0+s] = price + alpha*(wrk[r-1][0+s]-price);
         wrk[r][1+s] = (price - wrk[r][0+s])*(1-beta) + beta*wrk[r-1][1+s];
         wrk[r][2+s] = (wrk[r][0+s] + R*wrk[r][1+s]);
         wrk[r][3+s] = (wrk[r][2+s] - wrk[r-1][4+s])*MathPow((1-alpha),2) + MathPow(alpha,2)*wrk[r-1][3+s];
         wrk[r][4+s] = (wrk[r-1][4+s] + wrk[r][3+s]); 

   //
   //
   //
   //
   //

   return(wrk[r][4+s]);
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

double getPrice(enPrices price, const double& open[], const double& close[], const double& high[], const double& low[], int i, int bars)
{
   switch (price)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
   }
   return(0);
}